@@ -1,5 +1,4 @@
-/* $Id$ */
-/*-
+/*
  * Copyright (c) 2007 The Xfce Development Team
  *
  * This library is free software; you can redistribute it and/or
@@ -13,9 +12,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA
  */
 
 #if !defined (LIBXFCE4UI_INSIDE_LIBXFCE4UI_H) && !defined (LIBXFCE4UI_COMPILATION)
@@ -50,6 +49,16 @@ gboolean xfce_spawn_on_screen                  (GdkScreen    *screen,
                                                 const gchar  *startup_icon_name,
                                                 GError      **error);
 
+gboolean xfce_spawn_no_child                   (GdkScreen    *screen,
+                                                const gchar  *working_directory,
+                                                gchar       **argv,
+                                                gchar       **envp,
+                                                GSpawnFlags   flags,
+                                                gboolean      startup_notify,
+                                                guint32       startup_timestamp,
+                                                const gchar  *startup_icon_name,
+                                                GError      **error);
+
 gboolean xfce_spawn_command_line_on_screen     (GdkScreen    *screen,
                                                 const gchar  *command_line,
                                                 gboolean      in_terminal,
